#!/usr/bin/env python3
from pprint import pprint
from .agent_based_api.v1 import register, render, Service, Result, State, Metric


def getStateUpper(levels, value):
    warn, crit = levels
    if value >= crit:
        return State.CRIT
    if value >= warn:
        return State.WARN
    return State.OK


def getStateLower(levels, value):
    warn, crit = levels
    if value < crit:
        return State.CRIT
    if value < warn:
        return State.WARN
    return State.OK


def discover_mailcow_info(section):
    yield (Service())


def check_mailcow_info(params, section):
    for key in section:
        if key == "mailcow":
            # get thresholds
            levels_num_domains = params["levels_num_domains"]
            levels_num_mailboxes = params["levels_num_mailboxes"]
            levels_num_global_messages = params["levels_num_global_messages"]
            version = section[key]["version"]
            git_version = section[key]["git_version"]
            check_version_enabled = section[key]["check_version_enabled"]
            update_available = section[key]["update_available"]
            num_domains = section[key]["num_domains"]
            num_mailboxes = section[key]["num_mailboxes"]
            num_global_messages = section[key]["num_global_messages"]

            # create graphs for number of domains, mailboxes and messages
            yield (Metric("mc_num_domains", num_domains, levels=levels_num_domains))
            yield (
                Metric("mc_num_mailboxes", num_mailboxes, levels=levels_num_mailboxes)
            )
            yield (
                Metric(
                    "mc_num_global_messages",
                    num_global_messages,
                    levels=levels_num_global_messages,
                )
            )

            # create overall result
            if check_version_enabled:
                if update_available:
                    summary = f"Update available: Running version is {version}, Github version is: {git_version}"
                    state = State.WARN
                else:
                    summary = f"No update available: Running version is {version}, Github version is: {git_version}"
                    state = State.OK
            else:
                summary = f"Version is {version}, Update check is disabled"
                state = State.OK
            details = f"Mailcow version: {version}\nNumber of domains: {num_domains}\nNumber of mailboxes: {num_mailboxes}\nNumber of messages: {num_global_messages}"
            yield Result(state=state, summary=summary, details=details)

            # Create result for number of domains
            warn, crit = levels_num_domains
            state = getStateUpper((warn, crit), num_domains)
            notice = f"Number of domains: {num_domains}"
            if state != State.OK:
                yield (Result(state=state, notice=notice))

            # Create result for number of mailboxes
            warn, crit = levels_num_mailboxes
            state = getStateUpper((warn, crit), num_mailboxes)
            notice = f"Number of mailboxes: {num_mailboxes}"
            if state != State.OK:
                yield (Result(state=state, notice=notice))

            # Create result for number of global messages
            warn, crit = levels_num_global_messages
            state = getStateUpper((warn, crit), num_global_messages)
            notice = f"Number of messages: {num_global_messages}"
            if state != State.OK:
                yield (Result(state=state, notice=notice))


def parse_mailcow_info_section(string_table):
    # pprint(string_table)
    parsed_data = {
        "mailcow": {},
    }
    # we expect only one line
    line = string_table[0]
    version = line[0]
    num_domains = int(line[1])
    num_mailboxes = int(line[2])
    num_global_messages = int(line[3])
    git_version = line[4]
    update_available = line[5]
    if update_available == "True":
        update_available = True
    else:
        update_available = False
    check_version_enabled = line[6]
    if check_version_enabled == "True":
        check_version_enabled = True
    else:
        check_version_enabled = False
    parsed_data["mailcow"]["version"] = version
    parsed_data["mailcow"]["num_domains"] = num_domains
    parsed_data["mailcow"]["num_mailboxes"] = num_mailboxes
    parsed_data["mailcow"]["num_global_messages"] = num_global_messages
    parsed_data["mailcow"]["git_version"] = git_version
    parsed_data["mailcow"]["update_available"] = update_available
    parsed_data["mailcow"]["check_version_enabled"] = check_version_enabled
    # pprint(parsed_data)
    return parsed_data


register.agent_section(
    name="mailcow_info",
    parse_function=parse_mailcow_info_section,
)


register.check_plugin(
    name="mailcow_info",
    service_name="Mailcow Info",
    discovery_function=discover_mailcow_info,
    check_function=check_mailcow_info,
    check_default_parameters={
        "levels_num_domains": (100, 200),
        "levels_num_mailboxes": (500, 1000),
        "levels_num_global_messages": (100000, 250000),
    },
    check_ruleset_name="mailcow_info",
)
